\ 3iArt.fs - 2021-06-11---Fri---00:45
\ AE

\   Image Binarizer: https://www.dcode.fr/binary-image

\ format is column major (scan left to right along rows, then top to bottom)
\ a row is padded to by a byte multiple 
\ columns are padded to be byte multiple (for vertical oriented LCD)

\ User instructions:
\  1. save image to bmp form
\  2. use online tool \   Image Binarizer: https://www.dcode.fr/binary-image
\  3. upload image, custom threshold @ 25
\ threshold from black to white after conversion to greyscale using Rec 601 luma formula (30% red, 60% green, 10% blue):
\  Y=0.2989R+0.5870G+0.1140B 
\  4. binary chars 0=black, 1=white
\  5. paste into notepad++ .fs (GForth code file)
\  6. In NOTEPAD++: use macro Ctrl+Shift+Alt+P (pixels>b8c,) and record a macro to convert a padded row.  
\     Convert all rows bitmap image
\  7. create the Forth image_map in memory
\  8. create image descriptors as Forth constants...
\  9. use the memory stored bytes to extract and create the LCD vertical bytes and transmit to OLED
\ TODO: export and save the LCD byte codes to send the image directly without this step, e.g. for fast draw in hardware
\ TODO: read direct from bitmap file / write directly 

\ 20 lines of Forth code (+ Image Binarizer software, and Notepad++ macro) to display complex bitmap images on OLED screens
\ Using this can produce OLED ready graphics in 10-20 minutes depending on amount of touch-up needed

variable iw  \ image bit-width
variable ih  \ image bit-height
variable pw  \ padding bit-width to 8 bits
variable ph  \ padding bit-height to 8 bits
variable #b  \ #bytes in a command (less than MAX_I2C)

: pw@ pw c@ ;
: ph@ ph c@ ;
: #b@ #b c@ ;
: image-init ( iw ih -- bytes-per-transmission )  \ image width and height in bits
  2dup 
  ih c! iw c!
  8 /up 8 * ph c!  \ height/width padded to next highest byte
  8 /up 8 * pw c!
  2 begin pw@ over /mod #b c! swap 1 + dup rot 0 = swap 4 = or until drop \ find largest multiple of bytes less than MAX_I2C_PACKET
  ;
: lb# ( i j -- jmod lb# )  \ ex: 0 0 -> 0 0;   0 11 -> 3 1;  2 11 -> 3 13 ; 
    >r pw@ * r@ + 8 / r> 8 mod swap ; \ provides linear byte # that holds the desired bit
: pix@+ ( adr --  pix-val  adr' )  \ obtains pixel (or byte) from stored memory map, in downward access
    dup c@ swap pw@ 8 / + ;
: jmask8 ( j -- ) \ mask to pick off jth bit from left (j is 0 indexed, 0 to 7)
    8 1 - swap -   1 swap lshift ;  
: pickbit ( byte j -- bit )  \ form bit mask, apply to byte, and return bit value
    jmask8 and 0 > if 1 else 0 then ;  
: >oled ( image-map i j -- byte-lcd ) \ extra LCD graphic byte and send
    lb# rot + 
    8 0 do pix@+ -rot over pickbit swap rot loop 2drop \  find the column of bits from the respective bytes
    8 nrev >b8   \ create byte of data for OLED/LCD
;
: cmd_max?  ( #ct -- t/f ) #b@ mod 0 = ;

: bmp>oled ( w h image-map -- ) 
    -rot image-init pw@ . ." x " ph@ . cr
    ph@ 8 / 0 do cr i . ." :" cr         \ loop through rows
       pw@ 0 do dup j 8 * i >oled        \ loop through columns
                i cmd_max? if cr then dup hex. 1 oled-data    \  transmit byte to OLED (slow 1 byte at a time)
             loop 
       pw@ 128 < if .cr then cr          \ only write newline to OLED if row is less than width of OLED (128 pix)
    loop drop ;

\ =================================
: help-art
cr
cr ." help-art (3iArt.fs)"
cr 
cr ." include 3iForth.fs 3iI2C.fs 3iFont.fs 3iArt.fs"
cr ." include  3iArt-jaguar.fs   3iArt-butterfly.fs   3iArt-portraits.fs  3iArt-demo"

cr ." 1 open-com   rx"
cr ." oled-init   oled-clear"
cr ." oled-demo   photobook"
cr 
;

help-art


\  VERSIONS
\ V1 -  Initial prototype drwing
\ V2 -  First instance of photobook demo (4 photos).  Used direct data stream bytes 
\       instead of parsing and forming the commands from the 0/1 image
\       2021-06-21---Mon---02:08
\ V3 -  Finalized photbook - made 3iArt.fs modular and includeable in right sequence

\ NEXT STEPS
\ + streamline code/ inclusions/help
\ DONE + add images to family portrait - 3 more 
\ + move image commands to nano itself (C code) for untethered photobook operation
\ + setup on 1.5V AAA battery and micro booster -- should work
\ + figure out how to do the READ so that can position the row and mask on the new pixels as needed
\ + update ETCH -- pixel for pixel movement
\ + ETCH logic -- button press to lock the shape and connect with best curve (starting with line, then curve)
\ + I2C for Tiny85  (photobook, etch, beatmaster)
\ then back to piano light and input reading board

